@@ -1,4 +1,4 @@
 from .codec import encode, decode
-from .session import Session
+from .scoped_session import scoped_session
 from .cycle import Cycle
 from .drop import Drop
