@@ -1,7 +1,8 @@
 import base64
 import requests
 import time
-from io import IOBase
+from math import ceil
+from io import IOBase, BytesIO
 
 from .codec import decode, encode
 from .drop import Drop
@@ -9,6 +10,8 @@ from .drop import Drop
 
 class Cycle:
 
+    chunk_size = 16777216
+
     def __init__(self, session, open_data):
         self.session = session
         self.error = None
@@ -107,27 +110,59 @@ class Cycle:
             raise Exception('Invalid record name.')
         assert isinstance(stream, IOBase)
 
-        record_url = self.session.get_url('record/%s' % record.get('uid'))
-        record_res = requests.get(
-            record_url,
-            headers=self.session.get_headers(),
-            stream=True
-        )
-        record_res.raise_for_status()
+        file_size = record.get('byte_count')
+        chunk_count = ceil(file_size / Cycle.chunk_size)
 
-        for chunk in record_res.iter_content(chunk_size=16384):
-            stream.write(chunk)
+        for chunk_idx in range(chunk_count):
+            offset = Cycle.chunk_size * chunk_idx
+            length = min(Cycle.chunk_size, file_size - offset)
+
+            record_url = self.session.get_url('record/%s?offset=%s&length=%s' % (
+                record.get('uid'),
+                offset,
+                length
+            ))
+            record_res = requests.get(
+                record_url,
+                headers=self.session.get_headers()
+            )
+            record_res.raise_for_status()
+
+            stream.write(record_res.content)
 
 
     def push_record(self, name, stream):
         assert isinstance(stream, IOBase)
+
+        stream.seek(0, 2)
+        file_size = stream.tell()
+        chunk_count = ceil(file_size / Cycle.chunk_size)
+        stream.seek(0)
+
         cycle_url = self.session.get_url('cycle/%s' % self.uid)
-        cycle_res = requests.put(
-            cycle_url,
-            files = {name: stream},
+
+        for chunk_idx in range(chunk_count):
+            offset = Cycle.chunk_size * chunk_idx
+            length = min(Cycle.chunk_size, file_size - offset)
+
+            buf = BytesIO()
+            buf.write(stream.read(length))
+            buf.seek(0)
+
             headers=self.session.get_headers()
-        )
-        cycle_res.raise_for_status()
+            headers['Content-Range'] = 'bytes %s-%s/%s' % (
+                offset,
+                (offset+length)-1,
+                file_size
+            )
+
+            cycle_res = requests.put(
+                cycle_url,
+                data = {'offset': offset, 'length': length},
+                files = {name: buf},
+                headers = headers
+            )
+            cycle_res.raise_for_status()
 
 
     def commit(self):
