@@ -2,6 +2,7 @@ import requests
 from urllib.parse import urljoin
 
 from .cycle import Cycle
+from .renew_thread import RenewThread
 
 
 class Session:
@@ -52,6 +53,12 @@ class Session:
         pumps = pumps_data.get('pumps')
         self.pumps = [x.get('name') for x in pumps]
 
+        self.renew_thread = RenewThread(self)
+        self.renew_thread.start()
+
+    def exit(self):
+        self.renew_thread.stop()
+
     def open(self, pump):
         if pump not in self.pumps:
             raise Exception('Invalid pump name.')
@@ -68,3 +75,16 @@ class Session:
         open_res.raise_for_status()
         open_data = open_res.json()
         return Cycle(self, open_data)
+
+    def renew(self):
+        renew_res = requests.post(
+            self.get_url('renew'),
+            headers=self.get_headers()
+        )
+
+        renew_res.raise_for_status()
+        renew_data = renew_res.json()
+        self.token = renew_data.get('token')
+        user = renew_data.get('user')
+        self.username = user.get('name')
+        self.roles = [x.get('name') for x in user.get('roles')]
